<?php

/**
 * GridView Export Behavior
 * 
 * Usage:
 * <pre>
 * //(Step 1) Add this behavior to the controller like
  public function behaviors() {
  return array(
  'exportableGrid' => array(
  'class' => 'application.components.ExportableGridBehavior',
  'attributes' => array('user.name', 'title', 'date',),
  ));
  }
 * //(Step 2) On actionAdmin() , add this line before render('admin') method
  $this->exportGrid($model);
 * //(Step 3) On the view that renders the grid, add the Export button like this
  //$gridWidget=$this->widget('bootstrap.widgets.TbGridView', array( . . .
  $this->renderExportGridButton($gridWidget,'Export Grid Results',array('class'=>'btn btn-info pull-right'));
 * // (Optional) If you need to esport a very large dataset, (not recomended), you need to add before exportGrid()
  set_time_limit(0);
 * </pre>
 * @version 1.0
 * @author Geronimo Oñativia / http://www.estudiokroma.com
 * @link http://www.yiiframework.com/extension/exportablegridbehavior
 */
class ExportableGridBehavior extends CBehavior {

    public $buttonId = 'export-button';
    public $exportParam = 'exportGrid';
    public $csvDelimiter = ',';
    public $csvEnclosure = '"';
    public $filename = 'export.csv';
    public $attributes = array();

    /**
     * @param CModel $model A model with attribute filters already loaded
     * @param array $attributes Attributes of $model to be exported. null to use "attributes" property provided on the behaviour definition.
     */
    public function exportGrid($model, $attributes = null) {
        if (Yii::app()->request->getParam($this->exportParam, false)) {
            if ($attributes == null)
                $this->superExport($model, $this->attributes);
            else
                $this->superExport($model, $attributes);
            Yii::app()->end(0, false);
            exit(0);
        }
    }
    
    public function renderExportGridButton(CGridView $grid, $label = 'Export', $htmlOptions = array()) {
        $actionUrl = $this->getOwner()->createUrl($this->getOwner()->getAction()->getId());
        $fileDownloadUrl = $this->getOwner()->createAbsoluteUrl($this->getOwner()->getAction()->getId(), array($this->exportParam => '1'));
        if (!isset($htmlOptions['id'])) {
            $htmlOptions['id'] = $this->buttonId;
        }
        echo CHtml::link($label, '#',$htmlOptions);
        Yii::app()->clientScript->registerScript('exportgrid', "$('#" . $htmlOptions['id'] . "').on('click',function() { var data=$('#" . $grid->id . "').yiiGridView('getUrl').substring('{$actionUrl}'.length);window.open( '{$fileDownloadUrl}'+data,'_blank'); }); ");
    }

    private function superExport($model, $attributes) {
        // disable caching
        $now = gmdate("D, d M Y H:i:s");
        header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: {$now} GMT");
        // force download  
        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");
        // disposition / encoding on response body
        header("Content-Disposition: attachment;filename=\"{$this->filename}\"");
        header("Content-Transfer-Encoding: binary");

        $fp = fopen('php://output', 'w');
        fwrite($fp, "\xEF\xBB\xBF");
        /**
         * Put Headers 
         */
        $row = array();
        foreach ($attributes as $attr) {
            $row[] = $model->getAttributeLabel($attr);
        }
        fputcsv($fp, $row, $this->csvDelimiter, $this->csvEnclosure);
        /**
         * Put rows 
         */
        $provider = $model->search();

        $iterator = new CDataProviderIterator($provider, 150);
        foreach ($iterator as $m) {
            $row = array();
            foreach ($attributes as $attr) {
                $row[] = CHtml::value($m, $attr);
            }
            fputcsv($fp, $row, $this->csvDelimiter, $this->csvEnclosure);
        }
        fclose($fp);
    }

}

?>
